<?php

namespace Tvip\CoreBundle\Controller\ProviderApi;

require_once('ProviderApiException.php');

use Doctrine\Common\Inflector\Inflector;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\QueryBuilder;
use Doctrine\ORM\Tools\Pagination\Paginator;
use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Routing\ClassResourceInterface;
use FOS\RestBundle\Util\Codes;
use FOS\RestBundle\View\View;
use JMS\Serializer\DeserializationContext;
use JMS\Serializer\Serializer;
use Netsol\JMS\NetsolExclusionStrategy;
use Symfony\Bundle\FrameworkBundle\Routing\Router;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Tvip\CoreBundle\Entity\Admin;
use Tvip\CoreBundle\Entity\Provider;

class BaseRestController extends FOSRestController implements ClassResourceInterface
{
    protected $bundleName;
    protected $entityName;
    protected $className;
    /**
     * @var Admin
     */
    protected $admin;

    /**
     * @var Provider
     */
    protected $provider;
    /**
     * @var Router
     */
    protected $router;

    public function __construct(){
        $className=get_called_class();
        if (!preg_match("/(^.*)\\\(.*Bundle)\\\(.*)\\\(.*)Controller$/",$className,$matches)){
            throw new ProviderApiInternalErrorException("Could not get entity name from controller class '".$className."''");
        }
        //dump($matches);
        //return 0;
        $this->bundleName=$matches[1].$matches[2];
        $this->entityName=$matches[4];
        $this->className =$matches[1].'\\'.$matches[2].'\\Entity\\'.$matches[4];
    }

    /**
     * @return EntityManager
     */
    protected function getEntityManager(){
        $em=$this->getDoctrine()->getManager();
        return $em;
    }

    protected function getFullEntityName(){
        return $this->bundleName.':'.$this->entityName;
    }

    protected function getAlias(){
        return strtolower($this->entityName);
    }

    protected function view($data = null, $statusCode = null, array $headers = array()){
        $view=parent::view($data, $statusCode, $headers);
        $view->getSerializationContext()->addExclusionStrategy(new NetsolExclusionStrategy());
        $view->getSerializationContext()->setGroups('provider_api');
        $view->getSerializationContext()->setSerializeNull(true);
        return $view;
    }

    /**
     * @param string|null $entityClassName
     * @return \ReflectionProperty
     * @throws ProviderApiInternalErrorException
     */
    protected function getIdPropertyRefClass($entityClassName=null){
        /** @var EntityManager $em */
        $em=$this->getDoctrine()->getManager();

        $entityInfo=$em->getClassMetadata($entityClassName ? $entityClassName:$this->getFullEntityName());

        if (count($entityInfo->identifier)!=1) throw new ProviderApiInternalErrorException("Multiplay ID not supported for entity ".$entityInfo->name);

        $idPropertyRef = $entityInfo->reflClass->getProperty($entityInfo->identifier[0]);
        $idPropertyRef->setAccessible(true);

        return $idPropertyRef;
    }

    protected function getQueryBuilder(){

        $meta = $this->getEntityManager()->getClassMetadata($this->getFullEntityName());

        $qb=$this->getEntityManager()->createQueryBuilder()
            ->select($this->getAlias())
            ->from($this->getFullEntityName(),$this->getAlias())
        ;

        //explicitly get IDs of associated entities
        foreach ($meta->getAssociationMappings() as $assocName => $assoc) {
            if ($meta->isSingleValuedAssociation($assocName)){
                $qb->leftJoin($this->getAlias().'.'.$assocName,$assocName);
                $qb->addSelect($assocName);
            }
        }

        return $qb;
    }

    protected function getEntityById($id){
        $qb=$this->getQueryBuilder();

        $qb->where($this->getAlias().'.'.$qb->expr()->eq($this->getIdPropertyRefClass()->name,$id));
        $entity=$qb->getQuery()->getOneOrNullResult();

        if (!$entity) throw $this->createNotFoundException("Entity does not exist (".$this->getFullEntityName().", ID: ".$id.")");

        return $entity;
    }

    /**
     * Apply filters to param fetecher during list action (can be overwritten in child classes)
     *
     * @param array $criteria
     * @param QueryBuilder $qb
     */
    protected function applyFilters($criteria, QueryBuilder $qb){
        foreach ($criteria as $key=>$crit){
            if (is_null($crit)) continue;
            $key = Inflector::camelize($key);

            $ft = $this->getEntityManager()->getClassMetadata($this->className)->getTypeOfField($key);
            if($ft && $ft!='integer') $crit = '\''.$crit.'\'';
            $key = $this->getAlias().'.'.$key;


            $qb->andWhere($qb->expr()->eq($key,$crit));
        }

        $this->staticFilters($qb);
    }

    protected function staticFilters(QueryBuilder $qb){

    }

    /**
     * Get entity list
     *
     * @param ParamFetcher $paramFetcher
     * @return View
     */
    public function cgetAction(ParamFetcher $paramFetcher){
        $qb=$this->getQueryBuilder();

        $start    = $paramFetcher->get('start');
        $limit   = $paramFetcher->get('limit');
        $orderBy = $paramFetcher->get('sort');

        $criteria=$paramFetcher->all();
        unset($criteria['start'], $criteria['limit'], $criteria['sort']);
        $this->applyFilters($criteria, $qb);

        if (isset($orderBy) && count($orderBy)){
            //order was specified
            foreach ($orderBy as $field=>$direction){
                $qb->addOrderBy($this->getAlias().'.'.$field,$direction);
            }
        } else {
            // default order
            $idPropertyRef=$this->getIdPropertyRefClass();
            $qb->orderBy($this->getAlias().'.'.$idPropertyRef->name);
        }

        if (isset($start)){
            $qb->setFirstResult($start);
        }

        $paginator=NULL;

        if ($limit){
            $paginator=new Paginator($qb, $fetchJoinCollection = true);
            $qb->setMaxResults($limit);
        }


        //$entities=$qb->getQuery()->execute(NULL,Query::HYDRATE_ARRAY);
        $entities=$qb->getQuery()->execute();

        $arr=array('data' => $entities);
        $arr['total']=$paginator ? count($paginator) : count($entities);
        $arr['start']=(int)$start;
        $arr['limit']=(int)$limit;
        $view=$this->view($arr);

        return $view;
    }

    /**
     * Get single entity
     *
     * @param $id
     * @return View
     */
    public function getAction($id){

        $entity=$this->getEntityById($id);

        //$this->getEntityManager()->initializeObject($entity);

        $arr=array('data' => $entity);

        $view=$this->view($arr);

        return $view;
    }

    /**
     * @param Request $request
     * @return View
     * @throws ProviderApiInternalErrorException
     * @throws \Doctrine\ORM\Mapping\MappingException
     * @throws \Exception
     */
    public function postAction(Request $request){
        /** @var Serializer $serializer */
        $serializer=$this->get('jms_serializer');
        $object = json_decode($request->getContent());

        $entity=$serializer->deserialize(
            json_encode($object),
            $this->getEntityManager()->getClassMetadata($this->getFullEntityName())->name,
            $request->attributes->get('_format')
        );


        $entityMetaData = $this->getEntityManager()->getClassMetadata($this->getFullEntityName());

        $reflection = $this->getEntityManager()->getClassMetadata($this->getFullEntityName())->getReflectionClass();
        foreach($object as $name=>$value){
            if($entityMetaData->hasAssociation(Inflector::camelize($name))){
                $propertyValue = null;
                $associationClass = $entityMetaData->getAssociationMapping(Inflector::camelize($name));
                try{
                    $field = $reflection->getProperty(Inflector::camelize($name));

                }catch (\ReflectionException $re){
                    $field=null;
                    if($reflection->getParentClass()) $field = $reflection->getParentClass()->getProperty(Inflector::camelize($name));
                    if(!$field) throw  new \Exception('This and parent entity has not contain field '.$name);
                }


                if($value){
                    $propertyValue = $this->getEntityManager()->getRepository($associationClass['targetEntity'])->find($value);
                }
                $field->setAccessible(true);
                $field->setValue($entity,$propertyValue);
                $field->setAccessible(false);
            }
        }


        $this->getEntityManager()->persist($entity);

        $this->getEntityManager()->flush();

        return $this->routeRedirectView('get_'.Inflector::tableize($this->entityName),
            array(
                'id' => $this->getIdPropertyRefClass()->getValue($entity)
            ),
            Codes::HTTP_CREATED
        );
    }

    /**
     * Update entity
     *
     * @param $id
     * @param Request $request
     * @return View
     * @throws \Exception
     */
    public function putAction($id, Request $request){
        /** @var Serializer $serializer */
        $serializer=$this->get('jms_serializer');
        $object = json_decode($request->getContent());
        $object->id = (int)$id;
        $entity=$serializer->deserialize(
            json_encode($object),
            $this->getEntityManager()->getClassMetadata($this->getFullEntityName())->name,
            $request->attributes->get('_format'),
            new DeserializationContext()
        );

        $this->getIdPropertyRefClass()->setValue($entity,$id);
        $entityMetaData = $this->getEntityManager()->getClassMetadata($this->getFullEntityName());

        $reflection = $this->getEntityManager()->getClassMetadata($this->getFullEntityName())->getReflectionClass();
        $original = $this->getEntityById($id);
        //copy properties
        foreach($reflection->getProperties() as $property){
            $property->setAccessible(true);
            $property->setValue($entity,$property->getValue($original));
            $property->setAccessible(false);
        }
        //replace properties
        foreach($object as $name=>$value){
            $field = $reflection->getProperty(Inflector::camelize($name));
            if($entityMetaData->hasAssociation(Inflector::camelize($name)) && $value){
                $propertyValue = null;
                $associationClass = $entityMetaData->getAssociationMapping(Inflector::camelize($name));
                $propertyValue = $this->getEntityManager()->getRepository($associationClass['targetEntity'])->find($value);
            }else{
                $propertyValue = $value;

            }
            $field->setAccessible(true);
            $field->setValue($entity,$propertyValue);
            $field->setAccessible(false);
        }


        $this->getEntityManager()->merge($entity);
        $this->getEntityManager()->flush();

        return $this->routeRedirectView('get_'.Inflector::tableize($this->entityName),
            array(
                'id' => $this->getIdPropertyRefClass()->getValue($entity)
            ),
            Codes::HTTP_ACCEPTED
        );
    }

    /**
     * Delete entity
     *
     * @param $id
     * @return View
     */
    public function deleteAction($id){

        $entity=$this->getEntityById($id);
        $this->getEntityManager()->remove($entity);
        $this->getEntityManager()->flush();


        // There is a debate if this should be a 404 or a 204
        // see http://leedavis81.github.io/is-a-http-delete-requests-idempotent/
        return $this->routeRedirectView('get_'.Inflector::pluralize(Inflector::tableize($this->entityName)),
            array(), Codes::HTTP_NO_CONTENT);
    }


    public function setContainer(ContainerInterface $container = null)
    {
        parent::setContainer($container);
        $this->router = $this->container->get('router');
        $this->admin = $this->container->get('security.context')->getToken()->getUser();
        $this->provider = $this->admin->getProvider() ? $this->admin->getProvider() : null;
    }


    public function redirectToGetView($entity, $code){
        $className = get_class($entity);
        $classNameParts = explode('\\',$className);
        $entityName = end($classNameParts);
        $id = $this->getIdPropertyRefClass($this->bundleName.':'.$entityName)->getValue($entity);
        $redirectView = $this->routeRedirectView('get_'.strtolower($entityName),
            array(
                'id' => $id
            ),
            $code
        );
        return $redirectView;
    }


}